/*************************************************************************
 *
 *    Used with ICCARM and ARM.
 *
 *    File name   : oled_lib.c
 *    Description : OLED Library Routines
 *
 *    History :
 *    1. Date        : Jan, 2010
 *       Author      : Thorsten Erdmann
 *       Modified    : Peter Kampf
 *
 *    highlevel functions functions written by <thorsten.erdmann@gmx.de>
 *    $Revision: 14946 $
 **************************************************************************/
#include <stdbool.h>
#include <string.h>
#include "hardware.h"
#include "system.h"
#include "oled_cmd.h"
#include "oled_lib.h"

//define global variables for module oled_lib.c:
uint8 tx, ty;
uint8 bgcolor, fgcolor, txcolor;
const char *font;
const int x0=6;
uint8 g_strlengthMem0=0;
uint8 g_strlengthMem1=0;

//reference external variables & pointers:
extern const char *ptrmyfont;

/******************************************************************************
 * Function Name: OLED_Init()
 *
 * Description:
 *    This function inits both connected OLED displays
 * Calling Sequence: 
 *    void
 * Returns:
 *    void
 *****************************************************************************/
void OLED_Init()
{
const uint8 *pROMtable;
   
  /* ============ Reset Display: ============ */
  CLRB(OLED_RES); delay_ms(10);
  SETB(OLED_RES); delay_ms(200);
  /* ============ Disable CS: ============ */
  SETB(OLED_CS1);       // set CS
  SETB(OLED_CS2);       // set CS
  /* ============ init 1st OLED ============ */
  /* get init data from ROM table and config displays: */
  pROMtable= &OLED_initCode0[0];
  write_cmd_ROM(pROMtable,0);
  /* Display Off Config */
  write_cmd_ROM(&OLED_initCode_DispOff[0],0);
  delay_ms(50);
  /* ============ init 2nd OLED ============ */
  pROMtable= &OLED_initCode1[0];
  write_cmd_ROM(pROMtable,1);
  /* Display Off Config */
  write_cmd_ROM(&OLED_initCode_DispOff[0],1);
  delay_ms(50);
  /* ======= send DISPLAY ON Sequence ======= */
  write_cmd_ROM(&OLED_initCode_DispOn[0], 0);
  write_cmd_ROM(&OLED_initCode_DispOn[0], 1);
  /* ========= clear screens ========= */
  delay_ms(50);
  cls(0,0);
  cls(0,1);
}

// *************************************************************************
// Switches +12V Main Supply: ON
// *************************************************************************
void Mains12V_On(void)
{
  delay_ms(500);
  SETB(OLED_12V);
  delay_ms(50);
}

// *************************************************************************
// Switches +12V Main Supply: OFF
// *************************************************************************
void Mains12V_Off(void)
{
  CLRB(OLED_12V);
}

// *************************************************************************
// send a command byte to OLED controller
// *************************************************************************
void write_cmd(uint8 command, uint8 disp_nr) 
{
  //enable transmission according to selected display:
  if (disp_nr==0)  CLRB(OLED_CS1);
  else CLRB (OLED_CS2);
  CLRB(OLED_DC);      // D /C => Command => low
  //write to SPI:
  S0SPDR= command;
  //wait for SPI ready:
  while( !(S0SPSR & (1<<SPIF)));
  //disable transmission:
  SETB (OLED_CS1|OLED_CS2);  
}

// *************************************************************************
// send a data byte to OLED controller
// *************************************************************************
void write_data(uint8 databyte, uint8 disp_nr)
{
  DIS_Interrupt;
  //enable transmission according to selected display:
  if (disp_nr==0)  CLRB(OLED_CS1);
  else CLRB (OLED_CS2);
  SETB (OLED_DC);
  //write to SPI:
  S0SPDR= databyte;
  while( !(S0SPSR & (1<<SPIF)));
  //disable transmission:
  SETB (OLED_CS1|OLED_CS2);  
  SETB (OLED_CS1|OLED_CS2);  
  EN_Interrupt;
}

// *************************************************************************
// get ROM data from table
// *************************************************************************
void write_cmd_ROM(const uint8 *Str, uint8 disp_nr) 
{
  while (*Str!=0xff) 
  { 
    write_cmd(*Str, disp_nr);
    Str++;
  }
}

// *************************************************************************

// *************************************************************************
void cls(uint8 color, uint8 disp_nr)
{
	bgcolor=color;
	color+=color<<4;
	write_cmd(0x23, disp_nr);  // fill rectangle
	write_cmd(0x01, disp_nr);  // or not
	write_cmd(0x24, disp_nr);  // draw rectangle
	write_cmd(0x00, disp_nr);  // start column
	write_cmd(0x00, disp_nr);  // start row
	write_cmd(0x39, disp_nr);  // end column
	write_cmd(0x4f, disp_nr);  // end row
	write_cmd(color, disp_nr); // drawing gray scale pattern, left and right pixel
}

// *************************************************************************
// Deletes complete row
// example: cls_y(20,60,0);
// *************************************************************************
void cls_y(uint8 row_start, uint8 row_end, uint8 disp_nr)
{
	write_cmd(0x23, disp_nr);  // fill rectangle
	write_cmd(0x01, disp_nr);  // or not
	write_cmd(0x24, disp_nr);  // draw rectangle
	write_cmd(0x00, disp_nr);  // start column
	write_cmd(row_start, disp_nr);  // start row
	write_cmd(0x39, disp_nr);  // end column
	write_cmd(row_end, disp_nr);  // end row
	write_cmd(0, disp_nr); // drawing gray scale pattern, left and right pixel
}
// *************************************************************************
// Draw a horizontal line
// *************************************************************************
void hline(int x, int y, int l, uint8 disp_nr)
{
int numbytes;
uint8 c;
	if (l<=0) return;
	
	numbytes = (l+1)>>1;
	if (!(l&1) && (x&1)) numbytes++;
	
	write_cmd(0x15, disp_nr); // set column to x and x+w
	write_cmd((x>>1)+x0, disp_nr);
	write_cmd((x>>1)+x0+numbytes-1, disp_nr);
	
	write_cmd(0x75, disp_nr); // set row to ty + h
	write_cmd(y, disp_nr);
	write_cmd(y, disp_nr);
	
	c=(fgcolor&0x0f)|(fgcolor<<4);
	
	if (x&1)
	{
	  write_data(fgcolor+(bgcolor<<4), disp_nr);
	  numbytes--;
	}
	numbytes--;
	while (numbytes>0) 
	{
  	  write_data(c,disp_nr);
	  numbytes--;
	}
	if ((x+l)&1)
	  write_data(bgcolor+(fgcolor<<4), disp_nr);
	else
	  write_data(c, disp_nr);
}

// *************************************************************************
// Draw a vertical line
// *************************************************************************
void vline(int x, int y, int l, uint8 disp_nr)
{
uint8 color;
  if (l<=0) return;
  if (x&1) color=(fgcolor<<4) + bgcolor;
  else
  color=(bgcolor<<4) + fgcolor;//
  write_cmd(0x23, disp_nr);  // rectangle
  write_cmd(1, disp_nr);     // fill
  write_cmd(0x24, disp_nr);  // draw rectangle
  write_cmd((x>>1)+x0, disp_nr);    // start column
  write_cmd(y, disp_nr);	    // start row
  write_cmd((x>>1)+x0, disp_nr);    // end column
  write_cmd(y+l-1, disp_nr);	    // end row
  write_cmd(color, disp_nr);	    // drawing gray scale pattern, left and right pixel
}


// *************************************************************************
// Set Cursor to x,y
// *************************************************************************
void gotoxy(int x, int y)
{
	tx=x;
	ty=y;
}

// *************************************************************************
// Calculate a string size in pixel using the current font
// *************************************************************************
void strsize(const char *s, uint8 *w, uint8 *h)
{
const char  *ch, *pidx;
const char *ptrfont;
uint8 ascii_low, ascii_high;

  ptrfont  = &font[0]+1;
  ascii_low = *ptrfont++;
  ascii_high = *ptrfont++; // get lowest ascii code from font
  *h = *ptrfont++; // get character height from font
  // calculate string length in pixel
  ch=s; *w=0;
  while (*ch)
  {
    if ((*ch>=ascii_low) && (*ch<=ascii_high))
      pidx = ptrfont;   // save font pointer
      pidx+=(*ch-ascii_low)*3;
     *w+=*pidx;
     ch++;
  }
}

// *************************************************************************
// display a string
// *************************************************************************
void printstr(const char *s, uint8 disp_nr)
{
const char *ptrfont, *ptrhlp, *ptrfontidx;
const char *bm, *ch;
uint8 x, y, len, xc, ch_w, ch_h, bpr, ascii_low, ascii_high, bpp, mask, p, b;
int16   offset;

  ptrfont     = &font[0];
  bpp         = *ptrfont++; // get bits per pixel from font
  ascii_low   = *ptrfont++; // get lowest ascii code from font
  ascii_high  = *ptrfont++; // get lowest ascii code from font
  ch_h        = *ptrfont++; // get character height from font

  ptrfontidx = ptrfont;     // pointer to actual font data
  
  // calculate string length in pixel
  ch=s;
  len=0;
  while (*ch)
  {
    if ((*ch>=ascii_low) && (*ch<=ascii_high))
      ptrhlp = ptrfont;
      ptrhlp+=(*ch-ascii_low)*3;
      len+=*ptrhlp;
      ch++;
      //    len+=pgm_read_byte(r + (*ch-ascii_low) * 3);
  }
  // draw the entire line
  write_cmd(0x15, disp_nr); // set column to tx + w
  write_cmd((tx>>1)+x0, disp_nr);
  write_cmd((tx>>1)+x0+((len+1)>>1)-1, disp_nr);
	
  write_cmd(0x75, disp_nr); // set row to ty + h
  write_cmd(ty, disp_nr);
  write_cmd(ty+ch_h-1, disp_nr);
  delay_ms(2);
  for (y=0; y<ch_h; y++)
  {
    bm=0;
    ch=s;
    //point to size of char and cast arry to 16bit ptr:
//    ptrint = (int16*)&myfont[0];
    
    for (x=0; x<len; x++)
    {
      if (!bm) // we have no bitmap, so get the next character bitmap
      {
	// check if character is in the font
	if ((*ch>=ascii_low) && (*ch<=ascii_high))
	{
          ptrfont = ptrfontidx;           // point to first char at table
	  offset = (*ch-ascii_low) * 3;   // find the characters table entry
          ptrfont+= offset;               // read the character width
          ch_w   = *ptrfont;              // read the character width
          ptrfont++;
          //read 16bit bitmap address
	  offset = (int16)*ptrfont++; // read the bitmap address
	  offset |= (int16)*ptrfont<<8; // read the bitmap address

          switch (bpp)
	  {
            case 1:
              bpr    = ((ch_w+7)>>3);             // calculate the number of bytes per character row
	    break;
	    case 4:
	      bpr    = ((ch_w+1)>>1);             // calculate the number of bytes per character row
	    break;
          }
          bm = ptrfontidx + offset +bpr*y;
          p = *bm++;
          //	p      = pgm_read_byte(bm++);
	  mask   = 0x80;
    	  xc     = 0;
	}
        else
	  ch++;
      }
      if (bm)
      {
	switch (bpp)
	{
	  case 4:
	    if (!(x&1))
	    {
		b=0;
		if (!(xc&1))
		  b=p;
		else
		  b|=p<<4;
	    }
            else
	    {
		if (!(xc&1)) b|=p>>4;
		  write_data(b, disp_nr);
	    }
	    xc++;
	    if (xc>=ch_w)
	    {
		if (!(x&1) && (x==len-1)) write_data(b, disp_nr);
		bm=0;
		ch++;
	    }
	    else if (!(xc&1)) 
	    {
		p=*bm++;
	    }
	    break;
		case 1:
		  if (!(x&1))
		  {
		    b=0;
		    if (p & mask) 
		      b|=fgcolor<<4;
		    else
		      b|=bgcolor<<4;
		  }
		  else
		  {
		    if (p & mask) 
		      b|=fgcolor;
		    else
		      b|=bgcolor;
		      write_data(b, disp_nr);
		  }
		  mask>>=1;
		  xc++;
		  if (xc>=ch_w)
		  {
		    if (!(x&1) && (x==len-1)) write_data(b|bgcolor, disp_nr);
			bm=0;
		    ch++;
		  }
		  else if (!mask) 
		  {
          		p=*bm++;
			mask=0x80;
		  }
		  break;
		}
	    }
	 }
    }
    tx+=len;
}

// *************************************************************************
// draw a formatted text inside a box
// prototype: printfstr(2,0,MAXX,MAXY,XLEFT+YCENTERED,BORDER,0,acClock2,0);
// *************************************************************************
void printfstr(int x, int y, int w, int h, uint8 format, 
               uint8 border, uint8 fillcolor, const char *s, uint8 disp_nr)
{
uint8 strlength;
uint8 xs, ys, ch_w, ch_h;
uint8 bg=bgcolor;
  
  xs=tx;
  ys=ty;
  
	strsize(s, &ch_w, &ch_h);
	
	// set width and height if not given
	if (h==0)
	{
		h=ch_h;
		if (border&BORDERUPPER) h++;
		if (border&BORDERLOWER) h++;
	}
	if (w==0)
	{
		w=ch_w;
		if (border&BORDERLEFT)  w++;
		if (border&BORDERRIGHT) w++;
	}

	// calculate the starting coordinates
	if (format&XCENTERED)
	  tx=x+((w-ch_w)>>1);
	else 
	  if (format&XRIGHT)
	  {
	    tx=x+w-ch_w;
	    if (border&BORDERRIGHT) tx--;
	  }
	  else
	  {
	    tx=x+1;
	    if (border&BORDERLEFT) tx++;
	  }

       //calc Y Position:
        switch(format&(YCENTERED|YDOWN))
        {
          case YCENTERED:
            ty=y+((h-ch_h)>>1);
          break;
          case YDOWN:
	    ty=y+h-ch_h;
            if (border&BORDERLOWER) ty--;
          break;
          case YTOP:
  	    ty=y;
	    if (border&BORDERUPPER) ty++;
          break;
        }
	bgcolor=fillcolor;


        //clear bigfont screen in case of rangechange (i.e. from 3->2 digits or 2->1)
        if(font==ptrmyfont)
        {
          strlength=strlen(s);
          switch(disp_nr)
          {
            case 0:
              if(strlength>g_strlengthMem0) g_strlengthMem0=strlength;
              if(strlength<g_strlengthMem0)
              {
                cls_y(20,60,0);
                g_strlengthMem0=strlength;
              }
            break;
            case 1:
              if(strlength>g_strlengthMem1) g_strlengthMem1=strlength;
              if(strlength<g_strlengthMem1)
              {
                cls_y(20,60,1);
                g_strlengthMem1=strlength;
              }
            break;
          }
        }
        printstr(s, disp_nr);
    
	bgcolor=bg;
	tx=xs;
	ty=ys;
}

// *************************************************************************

// *************************************************************************
void printchar(char ch, uint8 disp_nr)
{
	char s[2];
	s[0]=ch;
	s[1]=0;
	printstr(s, disp_nr);
}

// *************************************************************************
// display a byte as a decimal number with two digits and leading zero
// *************************************************************************
void printbyte(uint8 i, uint8 disp_nr)
{
	if (i>99) printchar(i/100 + 0x30, disp_nr);
	i%=100;
	printchar(i/10 + 0x30, disp_nr);
	i%=10;
	printchar(i + 0x30, disp_nr);
}

