/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    File name   : timer.c
 *    Description : System specific timer routines
 *
 *    History :
 *    1. Date        : Jan, 2010
 *       Author      : Peter Kampf
 *
 *    $Revision: 14946 $
 **************************************************************************/
#include <stdbool.h>
#include "hardware.h"
#include "system.h"
#include "scheduler.h"
#include "timer.h"

volatile uint16 g_wdT0_STMR_NoDataTimeout;  //Softtimer: NoData timeout
volatile uint16 g_wdT0_STMR_GPSNOKTimeout;  //Softtimer: GPS NOK timeout
volatile uint16 g_wdT0_STMR_StartupTimeout;  //Softtimer: Startup timeout

/******************************************************************************
 * Function Name: timer0Init()
 *
 * Description: Init of Timer0 Vectors
 *    Delay
 *    Return: void
 *
 * Calling Sequence: 
 *    void uint32
 *
 * Returns:
 *    void
 *****************************************************************************/
extern void timer0_ISRVEC_init()
{
  // Disable all interrupts
  VICIntEnClear = 0xffffffff;

  // initialize the timer0 interrupt vector
  VICIntSelect &= ~VIC_BIT(VIC_TIMER0); // IRQ on TIMER0 line.
  VICIntEnable = VIC_BIT(VIC_TIMER0);    // TIMER0 interrupt enabled
  VICVectCntl1 = 0x20 | VIC_TIMER0; // Enable vector interrupt for TIMER0.
  VICVectAddr1 = (uint32)&timer0ISR;
  VICProtection = 0;
}

/******************************************************************************
 * Function Name: timer0init_start()
 *
 * Description:
 *    Delay
 *    Return: void
 *
 * Calling Sequence: 
 *    void uint32
 *
 * Returns:
 *    void
 *****************************************************************************/
extern void timer0_init_start()
{
  T0TCR= 0;        // Disable timer 0 (1 bit register!)
  T0CCR= 0;        // All Capture types disabled
  T0EMR= 0;        // All external match type disabled
  T0PC= 0;         // Prescaler is set to no division.
  T0MR0= PCLK/100;  // Count up to this value.
  T0MCR= 3;        // Reset and interrupt on MR0 (match register 0).
  T0TCR= 1;        // Enable timer 0  (1 bit register!)
}

//-----------------------------------------------
//function:	TMR_Manager(BYTE const,BYTE const)
//out :		NULL
//description:	administers reload of softtimers
//		associated to TIMER0
//-----------------------------------------------
extern void TMR_Manager(uint8 TMRname, uint8 State)
{
  DIS_Interrupt;
  switch (TMRname)
  {
     case C_TMR_NoDATAtimeout:
       if(State==C_TMR_ON)
       {
          g_wdT0_STMR_NoDataTimeout=C_PREVAL_TO_NODATA_EXPIRY; 
       }
       else
       {
         g_wdT0_STMR_NoDataTimeout=0;
       }
       break;
     case C_TMR_StartupTimeout:
       if(State==C_TMR_ON)
       {
          g_wdT0_STMR_StartupTimeout=C_PREVAL_TO_STARTUP_EXPIRY; 
       }
       else
       {
         g_wdT0_STMR_StartupTimeout=0;
       }
       break;
     case C_TMR_GPSNOKtimeout:
       if(State==C_TMR_ON)
       {
          g_wdT0_STMR_GPSNOKTimeout=C_PREVAL_TO_GPSNOK_EXPIRY; 
       }
       else
       {
         g_wdT0_STMR_GPSNOKTimeout=0;
       }
       break;
  }
  EN_Interrupt;
}

/******************************************************************************
 *
 * Function Name: timer0ISR()
 *
 * Description:
 *    This function implements the ISR for timer0.
 *
 * Calling Sequence: 
 *    void
 *
 * Returns:
 *    void
 *
 *****************************************************************************/
static void timer0ISR(void)
{
static volatile uint16 s_wdT0_STMR0=C_PREVAL_THREADSPEED0;
static volatile uint16 s_wdT0_STMR1=C_PREVAL_THREADSPEED1;
static volatile uint16 s_wdT0_STMR2=C_PREVAL_THREADSPEED2;
static volatile uint16 s_wdT0_SDBC0=C_PREVAL_DEBOUNCEVAL0; //debounce Softtimer0
static bool F_BUTTON1_STAT=FALSE;
uint32 wdInData; 

//Timer0 works with f=100Hz = 10ms period

// ----------------------------------------------------
//	Periodical Services
// ----------------------------------------------------

   //--- NMI Service THREAD0 period: 50ms
   if(--s_wdT0_STMR0==0)
   {
	s_wdT0_STMR0=C_PREVAL_THREADSPEED0;
        // issue nmi event (50ms tick) to schduler
	g_wdBitEvent|=C_BIT_EV_NMI_TICK_THREAD0;
   }

   if(--s_wdT0_STMR1==0)
   {
	s_wdT0_STMR1=C_PREVAL_THREADSPEED1;
        g_wdCount1++;
	g_wdBitEvent|=C_BIT_EV_TOGCOUNTER1;
   }

   if(--s_wdT0_STMR2==0)
   {
	s_wdT0_STMR2=C_PREVAL_THREADSPEED2;
        g_wdCount2++;
	g_wdBitEvent|=C_BIT_EV_ALIVE_TICK;
   }
   
// ----------------------------------------------------
//	one Shot / Suspending Services
// ----------------------------------------------------
   
   //Service StateChange/Fallback Timer (NoDATA):
    if(g_wdT0_STMR_NoDataTimeout!=0)
    {
       g_wdT0_STMR_NoDataTimeout--; 
       if(0==g_wdT0_STMR_NoDataTimeout)
       {
          // Timer expired, set StateChange 
         g_wdBitEvent|=C_BIT_EV_GOTO_NODATA;
       }
    }

   //Service StateChange/Fallback Timer (NoGPS):
    if(g_wdT0_STMR_GPSNOKTimeout!=0)
    {
       g_wdT0_STMR_GPSNOKTimeout--; 
       if(0==g_wdT0_STMR_GPSNOKTimeout)
       {
          // Timer expired, set StateChange 
         g_wdBitEvent|=C_BIT_EV_GOTO_GPS_NOK;
       }
    }

   //Service Startup Timer:
    if(g_wdT0_STMR_StartupTimeout!=0)
    {
       g_wdT0_STMR_StartupTimeout--; 
       if(0==g_wdT0_STMR_StartupTimeout)
       {
          // Timer expired, set StateChange 
         g_wdBitEvent|=C_BIT_EV_GOTO_IDLE;
       }
    }

   // ----------------------------------------------------
//	Debouncing Services using Timer0 freq
// ----------------------------------------------------
    //capture current switch state
    wdInData = IOPIN;
// ----------------------------------------------------
//      capture Button1 Switch Data
// ----------------------------------------------------
    if((wdInData&P16_BUTTON_1)==0)
    {
      //now key is pressed, count down:
      s_wdT0_SDBC0--;
      if(0==s_wdT0_SDBC0&&(!F_BUTTON1_STAT))
      {
        s_wdT0_SDBC0= C_PREVAL_DEBOUNCEVAL0;   
        // issue button event to ev.dispatcher
	g_wdBitEvent|=C_BIT_EV_BUTTON_APP;
        F_BUTTON1_STAT=TRUE;
      }
    }
    else {s_wdT0_SDBC0= C_PREVAL_DEBOUNCEVAL0; F_BUTTON1_STAT=FALSE;}
// ----------------------------------------------------
//      capture Button2 Switch Data
// ----------------------------------------------------

  T0IR = 0xff; // Clear timer 0 interrupt line.
}
/*-----------------------------------------------------------------------------*/
//       if(fMerker) A_LED1_OFF; else A_LED1_ON;
//       fMerker^=1;

