/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    File name   : uart.c
 *    Description : UART Routines
 *
 *    History :
 *    1. Date        : Feb, 2010
 *       Author      : Peter Kampf
 *
 *    $Revision: 00001 $
 **************************************************************************/
//#include "system.h"
#include <stdbool.h>
#include <string.h>
#include "hardware.h"
#include "nmea.h"
#include "types.h"
#include "scheduler.h"
#include "system.h"
#include "uart.h"

//define global variables:
volatile uint16 g_UARTRXstate;

/******************************************************************************
 *
 * Function Name: UART1_Init()
 *
 * Description:  
 *    This function initializes the UART for async mode
 *
 * Calling Sequence: 
 *    baudrate divisor - use UART_BAUD macro
 *    mode - see typical modes (uart.h)
 *    fmode - see typical fmodes (uart.h)
 *
 * Returns:
 *    void
 *
 * NOTE: uart1Init(UART_BAUD(9600), UART_8N1, UART_FIFO_8);
 *
 *****************************************************************************/
void UART1_Init(uint16 baud, uint8 mode, uint8 fmode)
{
  U1IER = 0x00;                         // disable all interrupts
  U1IIR;                                // clear interrupt ID
  U1RBR;                                // clear receive register
  U1LSR;                                // clear line status register
  U1FDR = 0x10;                         // DIVADDVAL=0; MULVAL=1

  // set the baudrate
  U1LCR = ULCR_DLAB_ENABLE;             // select divisor latches 
  U1DLL = (uint8)baud;                // set for baud low byte
  U1DLM = (uint8)(baud >> 8);         // set for baud high byte

  // set the number of characters and other
  // user specified operating parameters
  U1LCR = (mode & ~ULCR_DLAB_ENABLE);
  U1FCR = fmode;
 
  // initialize the interrupt vector
  VICIntSelect &= ~VIC_BIT(VIC_UART1);  // UART1 selected as IRQ
  VICIntEnable = VIC_BIT(VIC_UART1);    // UART1 interrupt enabled
  VICVectCntl2 = VIC_ENABLE | VIC_UART1;
  VICVectAddr2 = (uint32)UART1_ISR;    // address of the ISR
   // enable receiver interrupts
  U1IER |= UIER_ERBFI;
  g_UARTRXstate=0;
}
/******************************************************************************
 *
 * Function Name: UART1_ISR()
 *
 * Description:
 *    This function implements the ISR for UART1.
 *
 * Calling Sequence: 
 *    void
 *
 * Returns:
 *    void
 *
 *****************************************************************************/
void UART1_ISR(void)
{
  static volatile uint16 g_UARTNMEAframe=0;
  static char *pRXbuf, *ptr1;
  static char ac_prefixBuf[10];
  static uint8 prefixCnt=0;
  static uint8 charbuf;

  uint8 iid;

  // perform proper ISR entry so thumb-interwork works properly
  // loop until not more interrupt sources
  while (((iid = U1IIR) & UIIR_NO_INT) == 0)
  {
    // identify & process the highest priority interrupt
    switch (iid & UIIR_ID_MASK)
    {
      case UIIR_RLS_INT:                // Receive Line Status
        U1LSR;                          // read LSR to clear
        break;
      //UART1_RX_INT_MODE
      case UIIR_CTI_INT:                // Character Timeout Indicator
      case UIIR_RDA_INT:                // Receive Data Available
        do
        {
          charbuf = U1RBR;
          //********** look for '$'-designator:
          if (C_NMEA_DESIG_CHAR==charbuf) 
          {
            g_UARTRXstate=C_NMEA_CATCH_PREFIX;
            ptr1= ac_prefixBuf;
            prefixCnt=C_NMEA_DESIG_LENGTH;
          }            
          //********** Prefix Catch State:
         else if(g_UARTRXstate&C_NMEA_CATCH_PREFIX)
          {
            *ptr1++=charbuf;
            if(--prefixCnt==0)
            {
              ptr1=ac_prefixBuf;
              //set BufPtr to according NMEA string:
              if(strstr(ptr1,C_NMEA_GPRMC)!=NULL)
              {
                g_UARTRXstate=C_NMEA_SYNC;
                g_UARTNMEAframe|=C_RMC_RCVD;
                pRXbuf=pSTC_NMEABuf_RAW->RMC;
              }
              else if(strstr(ptr1,C_NMEA_GPGGA)!=NULL)
              {
                g_UARTRXstate=C_NMEA_SYNC;
                g_UARTNMEAframe|=C_GGA_RCVD;
                pRXbuf=pSTC_NMEABuf_RAW->GGA;
              }
              else if(strstr(ptr1,C_NMEA_GPVTG)!=NULL)
              {
                g_UARTRXstate=C_NMEA_SYNC;
                g_UARTNMEAframe|=C_VTG_RCVD;
                pRXbuf=pSTC_NMEABuf_RAW->VTG;
              }
              else if(strstr(ptr1,C_NMEA_GPGSA)!=NULL)
              {
                g_UARTRXstate=C_NMEA_SYNC;
                g_UARTNMEAframe|=C_GSA_RCVD;
                pRXbuf=pSTC_NMEABuf_RAW->GSA;
              }
              if(g_UARTRXstate&C_NMEA_SYNC)
              {
                //copy NMEAprefix onto left pos:
                strcpy(pRXbuf,ptr1);
                pRXbuf+=strlen(ptr1);
              }
              else
              {
                //nothing found:
                g_UARTRXstate=C_NMEA_IDLE;
              }
            }
          }
          //********** NMEA Sync State:
          else if(g_UARTRXstate&C_NMEA_SYNC)
          {
            if(charbuf!='\r')
               *pRXbuf++=charbuf;
            else
            {
              //end of NMEA string reached, check if all NMEA strings available:
              if((g_UARTNMEAframe&(C_RMC_RCVD+C_GGA_RCVD+C_VTG_RCVD+C_GSA_RCVD))==
                (C_RMC_RCVD+C_GGA_RCVD+C_VTG_RCVD+C_GSA_RCVD))
              {
                //send event to ev.dispatcher
        	g_wdBitEvent|=C_BIT_EV_NMEA_AVAIL;
                g_UARTNMEAframe=C_NMEA_IDLE; //reset all
              }
              g_UARTRXstate=C_NMEA_IDLE;
            }
          }
        }
        while (U1LSR & ULSR_RDR);
      break;
      case UIIR_MS_INT:                 // MODEM Status
        U1MSR;                          // read MSR to clear
      break;
      default:                          // Unknown
        U1LSR;
        U1RBR;
        U1MSR;
      break;
    }
  }
  VICVectAddr = 0x00000000;             // clear this interrupt from the VIC
}

